@@ -31,6 +31,8 @@ create a new instance from the same configuration.
 
 The game logic is implemented in :meth:`Match.attack` and
 :meth:`Match.end_turn`. Only these two methods change the match state.
+All executed actions are available in :attr:`Match.history` for e.g.
+match replays.
 
 The match loop is:
 
@@ -64,7 +66,7 @@ from . util import get_player_max_size
 
 State = namedtuple(
     'State',
-    'seat player winner area_players area_num_dice '
+    'num_steps seat player winner area_players area_num_dice '
     'player_areas player_num_areas player_max_size player_num_dice player_num_stock'
 )
 """
@@ -73,6 +75,7 @@ A convenience wrapper to access the current match state data at once. (`namedtup
 The (copied) value or (referenced) tuple properties of a :class:`Match`
 instance are:
 
+* :attr:`~Match.num_steps`
 * :attr:`~Match.seat`
 * :attr:`~Match.player`
 * :attr:`~Match.winner`
@@ -85,20 +88,33 @@ instance are:
 * :attr:`~Match.player_num_stock`
 
 The current State instance is available via :attr:`Match.state` and valid
-until a call of :meth:`Match.attack` or :meth:`Match.end_turn`.
+until a (successful) call of :meth:`Match.attack` or :meth:`Match.end_turn`.
+
+.. versionchanged:: 0.2.0
+   Added ``num_steps``.
 """
 
 Attack = namedtuple(
     'Attack',
+    'step '
     'from_player from_area from_dice from_sum_dice '
     'to_player to_area to_dice to_sum_dice '
-    'victory'
+    'victory '
+    'from_area_num_dice from_player_num_areas from_player_max_size from_player_num_dice '
+    'to_area_num_dice to_player_num_areas to_player_max_size to_player_num_dice'
 )
 """
 Information and result of an executed attack. (`namedtuple`)
 
 Attack instances are created in :meth:`Match.attack` and available via
-:attr:`Match.last_attack` afterwards.
+:attr:`Match.last_attack` and :attr:`Match.history` afterwards.
+
+.. attribute:: step
+   :type: int
+
+   The Attack's index in :attr:`Match.history`.
+
+   .. versionadded:: 0.2.0
 
 .. attribute:: from_player
    :type: int
@@ -144,14 +160,77 @@ Attack instances are created in :meth:`Match.attack` and available via
    :type: bool
 
    `True` if successful, `False` if defeated.
+
+.. attribute:: from_area_num_dice
+   :type: int
+
+   The number of dice placed on the attacking area (always `1` for standard game rules).
+
+   .. versionadded:: 0.2.0
+
+.. attribute:: from_player_num_areas
+   :type: int
+
+   The total number of areas occupied by the attacking player.
+
+   .. versionadded:: 0.2.0
+
+.. attribute:: from_player_max_size
+   :type: int
+
+   The maximal number of adjacent areas occupied by the attacking player.
+
+   .. versionadded:: 0.2.0
+
+.. attribute:: from_player_num_dice
+   :type: int
+
+   The total number of dice placed on the attacking player’s areas.
+
+   .. versionadded:: 0.2.0
+
+.. attribute:: to_area_num_dice
+   :type: int
+
+   The number of dice placed on the attacked area.
+
+   .. versionadded:: 0.2.0
+
+.. attribute:: to_player_num_areas
+   :type: int
+
+   The total number of areas occupied by the attacked player.
+
+   .. versionadded:: 0.2.0
+
+.. attribute:: to_player_max_size
+   :type: int
+
+   The maximal number of adjacent areas occupied by the attacked player.
+
+   .. versionadded:: 0.2.0
+
+.. attribute:: to_player_num_dice
+   :type: int
+
+   The total number of dice placed on the attacked player’s areas.
+
+   .. versionadded:: 0.2.0
 """
 
-Supply = namedtuple('Supply', 'player areas dice sum_dice num_stock')
+Supply = namedtuple('Supply', 'step player areas dice sum_dice area_num_dice player_num_stock')
 """
 The outcome of dice supply at the end of a player's turn. (`namedtuple`)
 
 Supply instances are created in :meth:`Match.end_turn` and available via
-:attr:`Match.last_supply` afterwards.
+:attr:`Match.last_supply` and :attr:`Match.history` afterwards.
+
+.. attribute:: step
+   :type: int
+
+   The Supply's index in :attr:`Match.history`.
+
+   .. versionadded:: 0.2.0
 
 .. attribute:: player
    :type: int
@@ -173,10 +252,20 @@ Supply instances are created in :meth:`Match.end_turn` and available via
 
    The sum of :attr:`dice`.
 
-.. attribute:: num_stock
+.. attribute:: area_num_dice
+   :type: tuple(int)
+
+   The number of dice placed on the areas in :attr:`areas`.
+
+   .. versionadded:: 0.2.0
+
+.. attribute:: player_num_stock
    :type: int
 
    The number of dice stored in the player's stock.
+
+   .. versionchanged:: 0.2.0
+      Renamed from "num_stock".
 """
 
 
@@ -229,6 +318,9 @@ class Match:
         self._last_attack = None
         self._last_supply = None
 
+        self.__history = []
+        self._history = None  # exposed (read only) mirror, created/updated only on request
+
         self._state = None  # for convenient full match state access/passing
         self._update_state()
 
@@ -242,6 +334,16 @@ class Match:
         """The :class:`State` instance of the current match state."""
         return self._state
 
+    @property
+    def num_steps(self):
+        """
+        The number of (successful) :meth:`attack` and :meth:`end_turn` calls. (`int`)
+
+        .. versionadded:: 0.2.0
+        """
+
+        return len(self.__history)
+
     @property
     def seat(self):
         """The current player seat index, `-1` if match is finished. (`int`)"""
@@ -312,6 +414,18 @@ class Match:
         """The :class:`Supply` instance created by the last (successful) call of :meth:`end_turn`."""
         return self._last_supply
 
+    @property
+    def history(self):
+        r"""
+        The sequence of all :class:`Attack`\s and :class:`Supply`\s so far. (`tuple(Attack/Supply)`)
+
+        .. versionadded:: 0.2.0
+        """
+
+        if self._history is None:
+            self._history = tuple(self.__history)
+        return self._history
+
     def set_from_area(self, area_idx):
         """
         Validate and set or unset the attacking area.
@@ -470,10 +584,17 @@ class Match:
         self._player_num_dice = tuple(self.__player_num_dice)
 
         self._last_attack = Attack(
+            self.num_steps,
             from_player_idx, self._from_area_idx, from_rand_dice, from_sum_dice,
             to_player_idx, self._to_area_idx, to_rand_dice, to_sum_dice,
-            victory
+            victory,
+            self.__area_num_dice[self._from_area_idx], self.__player_num_areas[from_player_idx],
+            self.__player_max_size[from_player_idx], self.__player_num_dice[from_player_idx],
+            self.__area_num_dice[self._to_area_idx], self.__player_num_areas[to_player_idx],
+            self.__player_max_size[to_player_idx], self.__player_num_dice[to_player_idx],
         )
+        self.__history.append(self._last_attack)
+        self._history = None
 
         self._update_state()
         self._from_area_idx = -1
@@ -524,14 +645,20 @@ class Match:
         self.__player_num_stock[player_idx] = num_stock
         self._player_num_stock = tuple(self.__player_num_stock)
 
-        area_supplies = tuple((a_idx, n_dice) for a_idx, n_dice in area_supplies.items() if n_dice)
+        area_supplies = tuple(
+            (a_idx, n_dice, self.__area_num_dice[a_idx])
+            for a_idx, n_dice in area_supplies.items() if n_dice
+        )
         self._last_supply = Supply(
-            player_idx,
+            self.num_steps, player_idx,
             tuple(area_supply[0] for area_supply in area_supplies),
             tuple(area_supply[1] for area_supply in area_supplies),
             sum(area_supply[1] for area_supply in area_supplies),
+            tuple(area_supply[2] for area_supply in area_supplies),
             num_stock
         )
+        self.__history.append(self._last_supply)
+        self._history = None
 
         while True:
             self._seat_idx += 1
@@ -548,7 +675,7 @@ class Match:
 
     def _update_state(self):
         self._state = State(
-            self._seat_idx, self.player, self._winner,
+            self.num_steps, self._seat_idx, self.player, self._winner,
             self._area_players, self._area_num_dice,
             self._player_areas, self._player_num_areas, self._player_max_size,
             self._player_num_dice, self._player_num_stock
